@@ -2,8 +2,12 @@ import types
 import dask
 from dask import is_dask_collection
 from dask.base import DaskMethodsMixin, tokenize
-from dask.core import quote
+from dask.core import quote, flatten
 from dask.highlevelgraph import HighLevelGraph
+from metagraph.core.plugin import ConcreteAlgorithm, Translator, ConcreteType
+from metagraph.core.compiler import optimize
+from .visualize import visualize
+from .tasks import DelayedAlgo, DelayedTranslate
 
 
 def single_key(seq):
@@ -63,6 +67,10 @@ class Placeholder(DaskMethodsMixin):
             dsk = {}
         self._dsk = dsk
 
+    @property
+    def key(self):
+        return self._key
+
     def __dask_graph__(self):
         return self._dsk
 
@@ -78,23 +86,58 @@ class Placeholder(DaskMethodsMixin):
     def __dask_postpersist__(self):
         return rebuild, (self.__class__, self._key)
 
+    @staticmethod
+    def __dask_optimize__(dsk, keys, **kwargs):
+        return optimize(dsk, output_keys=list(flatten(keys)), **kwargs)
+
+    def visualize(self, filename="mydask", format=None, optimize_graph=False, **kwargs):
+        return visualize(
+            self,
+            filename=filename,
+            format=format,
+            optimize_graph=optimize_graph,
+            **kwargs,
+        )
+
     @classmethod
-    def build(cls, key, func, args, kwargs=None):
+    def build(
+        cls,
+        key,
+        func,
+        args,
+        kwargs=None,
+        source_type=None,
+        result_type=None,
+        resolver=None,
+    ):
         dsk = {}
         new_args = []
         for arg in args:
             arg = taskify(arg, dsk)
             new_args.append(arg)
-        if kwargs:
-            new_kwargs_flat = []
-            for kw, val in kwargs.items():
-                val = taskify(val, dsk)
-                new_kwargs_flat.append([kw, val])
-            # Add this func to the task graph
-            dsk[key] = (ph_apply, func, new_args, (dict, new_kwargs_flat))
+
+        new_kwargs_flat = []
+        if kwargs is None:
+            kwargs = {}
+        for kw, val in kwargs.items():
+            val = taskify(val, dsk)
+            new_kwargs_flat.append([kw, val])
+        # Add this func to the task graph
+        if isinstance(func, ConcreteAlgorithm):
+            task_func = DelayedAlgo(func, result_type=result_type)
+            dsk[key] = (task_func, new_args, (dict, new_kwargs_flat))
+        elif isinstance(func, Translator):
+            task_func = DelayedTranslate(
+                func,
+                source_type=source_type,
+                result_type=result_type,
+                resolver=resolver,
+            )
+            dsk[key] = (task_func, new_args, (dict, new_kwargs_flat))
         else:
-            # Add this func to the task graph (no need for apply)
-            dsk[key] = (func,) + tuple(new_args)
+            task_func = ph_apply
+            dsk[key] = (task_func, func, new_args, (dict, new_kwargs_flat))
+
         return cls(key, dsk)
 
 
