@@ -2,8 +2,9 @@
 """
 import types
 import inspect
+import copy
 from functools import partial
-from typing import Callable, List, Dict, Set, Union, Any
+from typing import Callable, List, Dict, Set, Union, Any, Optional
 from .typecache import TypeCache, TypeInfo
 
 
@@ -103,6 +104,7 @@ class ConcreteType:
                 raise KeyError(f"{key} not allowed property of {self.__class__}")
             # maybe type check?
         self.props = dict(props)
+        self.__name__ = self.__class__.__name__
 
     def __init_subclass__(cls, *, abstract=None):
         """Enforce requirements on 'abstract' attribute"""
@@ -118,21 +120,6 @@ class ConcreteType:
         # Property caches live with each ConcreteType, allowing them to be easily accessible
         # separate from the Resolver
         cls._typecache = TypeCache()
-        # Ensure ConcreteType.method decorators are used in ConcreteType class
-        # They are intended only to be used in a Wrapper class
-        for name, val in cls.__dict__.items():
-            if getattr(val, "_is_type_method", False):
-                raise TypeError(
-                    "Invalid decorator: `ConcreteType.method` should only be used in a Wrapper class"
-                )
-            elif getattr(val, "_is_type_classmethod", False):
-                raise TypeError(
-                    "Invalid decorator: `ConcreteType.classmethod` should only be used in a Wrapper class"
-                )
-            elif getattr(val, "_is_type_staticmethod", False):
-                raise TypeError(
-                    "Invalid decorator: `ConcreteType.staticmethod` should only be used in a Wrapper class"
-                )
 
     @classmethod
     def get_typeinfo(cls, value):
@@ -165,7 +152,7 @@ class ConcreteType:
         for prop, val in props.items():
             if prop in aprops:
                 raise ValueError(
-                    '"Cannot preset "{prop}"; already set in abstract properties'
+                    f'Cannot preset "{prop}"; already set in abstract properties'
                 )
             aprops[prop] = val
 
@@ -431,8 +418,6 @@ class Wrapper(metaclass=MetaWrapper):
     The auto-created ConcreteType will be attached as `.Type` onto the wrapper class.
     """
 
-    _resolver = None
-
     def __init_subclass__(cls, *, abstract=None, register=True):
         if not register:
             cls._abstract = abstract
@@ -492,6 +477,17 @@ class Wrapper(metaclass=MetaWrapper):
         func._is_required_property = True
         return func
 
+    def translate(self, dst_type: Union[str, ConcreteType, "Wrapper"], **props):
+        import metagraph as mg
+
+        # This uses the current default resolver
+        return mg.translate(self, dst_type, **props)
+
+    def run(self, algo_name: str, *args, **kwargs):
+        import metagraph as mg
+
+        return mg.run(algo_name, self, *args, **kwargs)
+
 
 class Translator:
     """Converts from one concrete type to another, enforcing properties on the
@@ -499,14 +495,23 @@ class Translator:
 
     def __init__(self, func: Callable, include_resolver: bool):
         self.func = func
+        self.resolver = None
         self._include_resolver = include_resolver
         self.__name__ = func.__name__
         self.__doc__ = func.__doc__
         self.__wrapped__ = func
 
+    def copy_and_bind(self, resolver):
+        new_copy = copy.copy(self)
+        new_copy.resolver = resolver
+        return new_copy
+
     def __call__(self, src, *, resolver=None, **props):
+        if resolver is None:
+            resolver = self.resolver  # use bound resolver
+
         if self._include_resolver:
-            if resolver is None:
+            if resolver is None:  # pragma: no cover
                 raise ValueError("`resolver` is None, but is required by translator")
             if hasattr(resolver, "_resolver"):  # DaskResolver
                 resolver = resolver._resolver
@@ -595,32 +600,64 @@ class ConcreteAlgorithm:
         *,
         version: int = 0,
         include_resolver: bool = False,
+        compiler: Optional[str] = None,
     ):
         self.func = func
         self.abstract_name = abstract_name
         self.version = version
+        self.resolver = None
         self._include_resolver = include_resolver
+        self._compiler = compiler
+        self._compiled_func = None
         self.__name__ = func.__name__
         self.__doc__ = func.__doc__
         self.__wrapped__ = func
         self.__original_signature__ = inspect.signature(self.func)
         self.__signature__ = normalize_signature(self.__original_signature__)
 
+    def copy_and_bind(self, resolver):
+        new_copy = copy.copy(self)
+        new_copy.resolver = resolver
+        new_copy._compiled_func = None
+        return new_copy
+
     def __call__(self, *args, resolver=None, **kwargs):
+        if resolver is None:
+            resolver = self.resolver  # use bound resolver
+
+        if self._compiler is not None:
+            if self._compiled_func is not None:
+                func = self._compiled_func
+            elif resolver is not None:
+                func = resolver.compile_algorithm(
+                    self, literals={}
+                )  # TODO: pass literals here
+                self._compiled_func = func
+            else:
+                raise CompileError(
+                    f"Cannot call {self.__name__} without a 'resolver' argument to generate compiled function."
+                )
+        else:
+            func = self.func
+
         if self._include_resolver:
-            if resolver is None:
+            if resolver is None:  # pragma: no cover
                 raise ValueError(
                     "`resolver` is None, but is required by concrete algorithm"
                 )
             if hasattr(resolver, "_resolver"):  # DaskResolver
                 resolver = resolver._resolver
-            return self.func(*args, resolver=resolver, **kwargs)
+            return func(*args, resolver=resolver, **kwargs)
         else:
-            return self.func(*args, **kwargs)
+            return func(*args, **kwargs)
 
 
 def concrete_algorithm(
-    abstract_name: str, *, version: int = 0, include_resolver: bool = False
+    abstract_name: str,
+    *,
+    version: int = 0,
+    include_resolver: bool = False,
+    compiler: Optional[str] = None,
 ):
     def _concrete_decorator(func: Callable):
         return ConcreteAlgorithm(
@@ -628,7 +665,73 @@ def concrete_algorithm(
             abstract_name=abstract_name,
             version=version,
             include_resolver=include_resolver,
+            compiler=compiler,
         )
 
     _concrete_decorator.version = version
     return _concrete_decorator
+
+
+class CompileError(Exception):
+    """An error when compiling a concrete algorithm."""
+
+    pass
+
+
+class Compiler:
+    """A compiler plugin can fuse a task subgraph into a single operation.
+
+    Generally compilation is done with a JIT compiler, but that is not required.
+    Compilers will only be asked to compile task graphs which contain concrete
+    implementations which are tagged as compilable with this compiler.
+    """
+
+    def __init__(self, name: str):
+        """Create a compiler with given name.
+
+        The name will be used as a key to match with the compilable concrete
+        implementations.
+        """
+        self.name = name
+
+    def initialize_runtime(self):
+        """Initialize any resources or devices required before this compiler
+        target can be used.
+
+        If called multiple times, calls after the first should do nothing.
+        """
+        pass
+
+    def teardown_runtime(self):
+        """Shutdown and free any runtime resources reserved by this compiler.
+
+        Not guaranteed to be called before process exit, and should be safe to
+        call multiple times.
+        """
+        pass
+
+    def compile_algorithm(
+        self, algo: ConcreteAlgorithm, literals: Dict[str, Any] = None
+    ) -> Callable:
+        """Compile concrete algorithm to a callable that matches its function signature.
+        
+        If any literal values (Python ints, strings, etc) should be frozen at compile time,
+        they are passed in the literals dict.
+
+        Implementation should automatically initialize runtime if needed and not already.
+        """
+        raise NotImplementedError(
+            "all compiler plugins must implement compile_algorithm()"
+        )
+
+    def compile_subgraph(
+        self, subgraph: Dict, inputs: List[str], output: str
+    ) -> Callable:
+        """Compile a subgraph of compilable functions into a single callable with
+        inputs in the order listed, returning output value.
+
+        Raises CompileError if unsuccessful.
+        """
+        raise NotImplementedError(
+            "all compiler plugins must implement compile_subgraph()"
+        )
