@@ -142,8 +142,6 @@ def get_plugins(resolver, **kwargs):
                         ][
                             concrete_type_name
                         ] = {}  # TODO add other useful information here
-            else:
-                pass  # TODO consider showing algorithm versions
     return result
 
 
@@ -202,7 +200,7 @@ def list_translators(resolver, source_type, **kwargs):
             trans_keys = getattr(resolver.plugins, plugin).translators
             if (src, dst) in trans_keys:
                 break
-        else:
+        else:  # pragma: no cover
             plugin = "Unknown"
         trans_info = [
             ("type", "translator"),
@@ -284,13 +282,13 @@ def list_algorithm_params(resolver, abstract_pathname: str, **kwargs):
             combo_type = " or ".join(r["type"] for r in resolved)
             choices = [c for r in resolved for c in r["choices"]]
             if p.optional:
-                combo_type += " or NoneType"  # should this really be type(NoneType)? Python doesn't have a concept of abstract types
+                combo_type += " or None"
                 choices.append("NoneType")
             result = OrderedDict([("type", combo_type), ("choices", choices)])
         elif issubclass(p_class, AbstractType):
             choices = list(types[p_class.__name__]["children"].keys())
             result = OrderedDict([("type", p_class.__name__), ("choices", choices)])
-        elif getattr(p, "__origin__", None) == collections.abc.Callable:
+        elif typing.get_origin(p) == collections.abc.Callable:
             result = OrderedDict([("type", p._name), ("choices", [p._name])])
         elif p is typing.Any:
             result = OrderedDict([("type", "Any"), ("choices", ["Any"])])
@@ -305,8 +303,8 @@ def list_algorithm_params(resolver, abstract_pathname: str, **kwargs):
         params[pname] = resolve_parameter(p.annotation)
 
     returns = []
-    if getattr(sig.return_annotation, "__origin__", None) is tuple:
-        for ret in sig.return_annotation.__args__:
+    if typing.get_origin(sig.return_annotation) is tuple:
+        for ret in typing.get_args(sig.return_annotation):
             returns.append(resolve_parameter(ret))
     else:
         returns.append(resolve_parameter(sig.return_annotation))
