@@ -35,7 +35,8 @@ except ImportError:  # pragma: no cover
 try:
     import grblas as _grblas
 
-    if _grblas.__version__ < "v1.3.2":
+    # older versions of grblas had a "v" in their version string
+    if _grblas.__version__.strip("v") < "1.3.2":  # pragma: no cover
         warnings.warn(
             f"grblas {_grblas.__version__} is installed, but >=v1.3.2 is required"
         )
@@ -64,10 +65,10 @@ registry = metagraph.PluginRegistry("core")
 
 
 def find_plugins():
-    from . import graphblas, networkx, numpy, pandas, python, scipy
+    from . import core, graphblas, networkx, numpy, pandas, python, scipy
 
     # Default Plugins
-    registry.register_from_modules(metagraph.types, metagraph.algorithms)
+    registry.register_from_modules(core)
     registry.register_from_modules(graphblas, name="core_graphblas")
     registry.register_from_modules(networkx, name="core_networkx")
     registry.register_from_modules(numpy, name="core_numpy")
