@@ -1,6 +1,7 @@
 import metagraph as mg
 from metagraph import concrete_algorithm, NodeID
 from metagraph.plugins import has_networkx, has_community, has_pandas
+from metagraph.plugins.core import exceptions
 from typing import Tuple, Any, Callable
 import random
 
@@ -16,9 +17,14 @@ if has_networkx:
     def nx_pagerank(
         graph: NetworkXGraph, damping: float, maxiter: int, tolerance: float
     ) -> PythonNodeMapType:
-        pagerank = nx.pagerank(
-            graph.value, alpha=damping, max_iter=maxiter, tol=tolerance, weight=None
-        )
+        try:
+            pagerank = nx.pagerank(
+                graph.value, alpha=damping, max_iter=maxiter, tol=tolerance, weight=None
+            )
+        except nx.exception.PowerIterationFailedConvergence:
+            raise exceptions.ConvergenceError(
+                f"failed to converge within {maxiter} iterations"
+            )
         return pagerank
 
     @concrete_algorithm("centrality.katz")
@@ -333,17 +339,19 @@ if has_networkx:
     def nx_graph_degree(
         graph: NetworkXGraph, in_edges: bool, out_edges: bool
     ) -> PythonNodeMapType:
+        g = graph.value
+        if not in_edges and not out_edges:
+            return {n: 0 for n in g.nodes()}
+        if not g.is_directed():
+            return dict(g.degree())
         if in_edges and out_edges:
-            ins = graph.value.in_degree()
-            outs = graph.value.out_degree()
-            d = {n: ins[n] + o for n, o in outs}
-        elif in_edges:
-            d = dict(graph.value.in_degree())
-        elif out_edges:
-            d = dict(graph.value.out_degree())
-        else:
-            d = {n: 0 for n in graph.value.nodes()}
-        return d
+            ins = g.in_degree()
+            outs = g.out_degree()
+            return {n: ins[n] + o for n, o in outs}
+        if in_edges:
+            return dict(g.in_degree())
+        if out_edges:
+            return dict(g.out_degree())
 
     @concrete_algorithm("util.graph.aggregate_edges")
     def nx_graph_aggregate_edges(
@@ -406,7 +414,8 @@ if has_networkx:
 
     @concrete_algorithm("subgraph.sample.node_sampling")
     def nx_node_sampling(graph: NetworkXGraph, p: float) -> NetworkXGraph:
-        if p <= 0 or p > 1:
+        # TODO: move this check into the abstract algorithm layer
+        if p <= 0 or p > 1:  # pragma: no cover
             raise ValueError(f"Probability `p` must be between 0 and 1, found {p}")
         aprops = NetworkXGraph.Type.compute_abstract_properties(
             graph, {"node_type", "edge_type"}
@@ -436,7 +445,8 @@ if has_networkx:
 
     @concrete_algorithm("subgraph.sample.edge_sampling")
     def nx_edge_sampling(graph: NetworkXGraph, p: float) -> NetworkXGraph:
-        if p <= 0 or p > 1:
+        # TODO: move this check into the abstract algorithm layer
+        if p <= 0 or p > 1:  # pragma: no cover
             raise ValueError(f"Probability `p` must be between 0 and 1, found {p}")
         aprops = NetworkXGraph.Type.compute_abstract_properties(
             graph, {"node_type", "edge_type"}
@@ -468,7 +478,8 @@ if has_networkx:
         Totally Induced Edge Sampling method
         https://docs.lib.purdue.edu/cgi/viewcontent.cgi?article=2743&context=cstech
         """
-        if p <= 0 or p > 1:
+        # TODO: move this check into the abstract algorithm layer
+        if p <= 0 or p > 1:  # pragma: no cover
             raise ValueError(f"Probability `p` must be between 0 and 1, found {p}")
         aprops = NetworkXGraph.Type.compute_abstract_properties(
             graph, {"node_type", "edge_type"}
@@ -514,11 +525,15 @@ if has_networkx:
         When resetting the walk, if start_node is specified, always reset to this node. If not specified, every reset
             picks a new node in the graph at random.
         """
-        if jump_probability <= 0 or jump_probability > 1:
+        # TODO: move this check into the abstract algorithm layer
+        if jump_probability <= 0 or jump_probability > 1:  # pragma: no cover
             raise ValueError(
                 f"`jump_probability` must be between 0 and 1, found {jump_probability}"
             )
-        if num_steps is None and num_nodes is None and num_edges is None:
+        # TODO: move this check into the abstract algorithm layer
+        if (
+            num_steps is None and num_nodes is None and num_edges is None
+        ):  # pragma: no cover
             raise ValueError(
                 "Must specify at least one of num_steps, num_nodes, or num_edges"
             )
