@@ -1,8 +1,8 @@
 import numpy as np
 from typing import Set, Dict, Any
 from metagraph import ConcreteType, dtypes
-from metagraph.types import DataFrame, EdgeSet, EdgeMap
-from metagraph.wrappers import EdgeSetWrapper, EdgeMapWrapper
+from ..core.types import DataFrame, EdgeSet, EdgeMap
+from ..core.wrappers import EdgeSetWrapper, EdgeMapWrapper
 from metagraph.plugins import has_pandas
 import math
 
@@ -30,7 +30,7 @@ if has_pandas:
                 pd.testing.assert_frame_equal(
                     obj1, obj2, check_like=True, rtol=rel_tol, atol=abs_tol
                 )
-            else:
+            else:  # pragma: no cover
                 digits_precision = round(-math.log(rel_tol, 10))
                 pd.testing.assert_frame_equal(
                     obj1, obj2, check_like=True, check_less_precise=digits_precision
@@ -64,7 +64,7 @@ if has_pandas:
                     .set_index([dst_label, src_label])
                     .index
                 )
-                dups = self.index & rev_index
+                dups = self.index.intersection(rev_index)
                 if len(dups) > 0:
                     raise ValueError(
                         f"is_directed=False, but duplicate edges found: {dups}"
@@ -73,15 +73,15 @@ if has_pandas:
         @property
         def num_nodes(self):
             src_nodes, dst_nodes = self.index.levels
-            return len(src_nodes | dst_nodes)
+            return len(src_nodes.union(dst_nodes))
 
-        def copy(self):
-            return PandasEdgeSet(
-                self.value.copy(),
-                self.src_label,
-                self.dst_label,
-                is_directed=self.is_directed,
-            )
+        # def copy(self):
+        #     return PandasEdgeSet(
+        #         self.value.copy(),
+        #         self.src_label,
+        #         self.dst_label,
+        #         is_directed=self.is_directed,
+        #     )
 
         class TypeMixin:
             @classmethod
@@ -115,17 +115,17 @@ if has_pandas:
                 g2 = obj2.value
                 assert len(g1) == len(g2), f"{len(g1)} != {len(g2)}"
                 if aprops1["is_directed"]:
-                    assert len(obj1.index & obj2.index) == len(
+                    assert len(obj1.index.intersection(obj2.index)) == len(
                         obj1.index
-                    ), f"edge mismatch {obj1.index ^ obj2.index}"
+                    ), f"edge mismatch {obj1.index.symmetric_difference(obj2.index)}"
                 else:  # undirected
                     rev1 = g1.set_index([obj1.dst_label, obj1.src_label])
                     rev2 = g2.set_index([obj2.dst_label, obj2.src_label])
-                    full1 = obj1.index | rev1.index
-                    full2 = obj2.index | rev2.index
-                    assert len(full1 & full2) == len(
+                    full1 = obj1.index.union(rev1.index)
+                    full2 = obj2.index.union(rev2.index)
+                    assert len(full1.intersection(full2)) == len(
                         full1
-                    ), f"edge mismatch {full1 ^ full2}"
+                    ), f"edge mismatch {full1.symmetric_difference(full2)}"
 
     class PandasEdgeMap(EdgeMapWrapper, abstract=EdgeMap):
         """
@@ -175,7 +175,7 @@ if has_pandas:
                     .set_index([dst_label, src_label])
                     .index
                 )
-                dups = self.index & rev_index
+                dups = self.index.intersection(rev_index)
                 if len(dups) > 0:
                     raise ValueError(
                         f"is_directed=False, but duplicate edges found: {dups}"
@@ -184,16 +184,16 @@ if has_pandas:
         @property
         def num_nodes(self):
             src_nodes, dst_nodes = self.index.levels
-            return len(src_nodes | dst_nodes)
+            return len(src_nodes.union(dst_nodes))
 
-        def copy(self):
-            return PandasEdgeMap(
-                self.value.copy(),
-                self.src_label,
-                self.dst_label,
-                self.weight_label,
-                is_directed=self.is_directed,
-            )
+        # def copy(self):
+        #     return PandasEdgeMap(
+        #         self.value.copy(),
+        #         self.src_label,
+        #         self.dst_label,
+        #         self.weight_label,
+        #         is_directed=self.is_directed,
+        #     )
 
         class TypeMixin:
             @classmethod
@@ -218,14 +218,7 @@ if has_pandas:
                             neg_weights = None
                         else:
                             min_val = obj.value[obj.weight_label].min()
-                            if min_val < 0:
-                                neg_weights = True
-                            else:
-                                min_val = obj.value[obj.weight_label].min()
-                                if min_val < 0:
-                                    neg_weights = True
-                                else:
-                                    neg_weights = False
+                            neg_weights = min_val < 0
                         ret[prop] = neg_weights
 
                 return ret
@@ -249,17 +242,17 @@ if has_pandas:
                 assert len(g1) == len(g2), f"{len(g1)} != {len(g2)}"
 
                 if aprops1["is_directed"]:
-                    assert len(obj1.index & obj2.index) == len(
+                    assert len(obj1.index.intersection(obj2.index)) == len(
                         obj1.index
-                    ), f"edge mismatch {obj1.index ^ obj2.index}"
+                    ), f"edge mismatch {obj1.index.symmetric_difference(obj2.index)}"
                 else:  # undirected
                     rev1 = g1.set_index([obj1.dst_label, obj1.src_label])
                     rev2 = g2.set_index([obj2.dst_label, obj2.src_label])
-                    full1 = obj1.index | rev1.index
-                    full2 = obj2.index | rev2.index
-                    assert len(full1 & full2) == len(
+                    full1 = obj1.index.union(rev1.index)
+                    full2 = obj2.index.union(rev2.index)
+                    assert len(full1.intersection(full2)) == len(
                         full1
-                    ), f"edge mismatch {full1 ^ full2}"
+                    ), f"edge mismatch {full1.symmetric_difference(full2)}"
 
                 # Ensure dataframes are indexed the same
                 if not (obj1.index == obj2.index).all():
