@@ -12,16 +12,19 @@ from metagraph import (
 from metagraph.core.plugin_registry import PluginRegistry
 from metagraph.core.resolver import (
     Resolver,
+    _ResolverRegistrar,
     Namespace,
     Dispatcher,
     NamespaceError,
     AlgorithmWarning,
+    _SignatureModifier,
 )
 from metagraph.core.planning import MultiStepTranslator, AlgorithmPlan
 from metagraph import config
 import typing
-from typing import Tuple, List, Dict, Any
-from collections import OrderedDict
+from typing import Tuple, Union, List, Dict, Any, Optional
+from metagraph.core import typing as mgtyping
+from collections import OrderedDict, defaultdict
 
 from .util import site_dir, example_resolver
 
@@ -51,6 +54,8 @@ def test_namespace():
     with pytest.raises(NamespaceError, match="Name already registered"):
         ns._register("A.B.c", 4)
 
+    assert ns.to_dict() == {"A": {"B": {"c": 3, "d": "test",}, "other": 1.5,}}
+
 
 def test_dispatcher(example_resolver):
     ns = Dispatcher(example_resolver, "power")
@@ -102,6 +107,10 @@ def test_register_errors():
     def c1_to_c2(src: Concrete1, **props) -> Concrete2:  # pragma: no cover
         pass
 
+    @translator
+    def c2_to_c1(src: Concrete2, **props) -> Concrete1:  # pragma: no cover
+        pass
+
     registry.register(c1_to_c2)
     with pytest.raises(ValueError, match="has unregistered abstract type "):
         res.register(registry.plugins)
@@ -111,8 +120,14 @@ def test_register_errors():
         ValueError, match="translator destination type .* has not been registered"
     ):
         res.register(registry.plugins)
+    with pytest.raises(
+        ValueError, match="translator source type .* has not been registered"
+    ):
+        res.register(
+            {"test_register_errors_default_plugin": {"translators": {c2_to_c1}}}
+        )
 
-    # Fresh start -- too much baggage of things partially registered above
+    # Fresh start
     res = Resolver()
 
     registry.register(Abstract2)
@@ -137,20 +152,20 @@ def test_register_errors():
         res.register(my_algo2_registry.plugins)
 
     @abstract_algorithm("testing.bad_input_type")
-    def my_algo_bad_list_input_type(a: List) -> Resolver:  # pragma: no cover
+    def my_algo_bad_list_input_type(a: List) -> int:  # pragma: no cover
         pass
 
     @abstract_algorithm("testing.bad_output_type")
     def my_algo_bad_output_type(a: Abstract1) -> res:  # pragma: no cover
         pass
 
-    @abstract_algorithm("testing.bad_compound_output_type")
-    def my_algo_bad_compound_list_output_type(
-        a: Abstract1,
-    ) -> Tuple[List, List]:  # pragma: no cover
-        pass
+    # @abstract_algorithm("testing.bad_compound_output_type")
+    # def my_algo_bad_compound_list_output_type(
+    #     a: Abstract1,
+    # ) -> Tuple[List, List]:  # pragma: no cover
+    #     pass
 
-    with pytest.raises(TypeError, match="must pass exactly one parameter to List"):
+    with pytest.raises(TypeError, match="Found an empty list of types for kind=List"):
         registry = PluginRegistry("test_register_errors_default_plugin")
         registry.register(my_algo_bad_list_input_type)
         res_tmp = Resolver()
@@ -162,11 +177,14 @@ def test_register_errors():
         res_tmp = Resolver()
         res_tmp.register(registry.plugins)
 
-    with pytest.raises(TypeError, match="must pass exactly one parameter to List"):
-        registry = PluginRegistry("test_register_errors_default_plugin")
-        registry.register(my_algo_bad_compound_list_output_type)
-        res_tmp = Resolver()
-        res_tmp.register(registry.plugins)
+    # with pytest.raises(
+    #     TypeError,
+    #     match="return type may not be an instance of type <class 'typing.TypeVar'>",
+    # ):
+    #     registry = PluginRegistry("test_register_errors_default_plugin")
+    #     registry.register(my_algo_bad_compound_list_output_type)
+    #     res_tmp = Resolver()
+    #     res_tmp.register(registry.plugins)
 
     @abstract_algorithm("testing.bad_input_type")
     def my_algo_bad_dict_input_type(a: Dict) -> Resolver:  # pragma: no cover
@@ -221,6 +239,15 @@ def test_register_errors():
         registry.register(conc_algo_with_defaults)
         res.register(registry.plugins)
 
+    @concrete_algorithm("testing.myalgo", include_resolver=True)
+    def conc_algo_with_resolver_default(a: Concrete1, *, resolver=14) -> Concrete2:
+        return a
+
+    with pytest.raises(TypeError, match='"resolver" cannot have a default'):
+        registry = PluginRegistry("test_register_errors_default_plugin")
+        registry.register(conc_algo_with_resolver_default)
+        res.register(registry.plugins)
+
     @abstract_algorithm("testing.multi_ret")
     def my_multi_ret_algo() -> Tuple[int, int, int]:  # pragma: no cover
         pass
@@ -247,13 +274,139 @@ def test_register_errors():
         return 12
 
     with pytest.raises(
-        TypeError, match='argument "x" does not match abstract function signature'
+        TypeError,
+        match='argument "x": .* does not match typing.Any in abstract signature',
     ):
         registry = PluginRegistry("test_register_errors_default_plugin")
         registry.register(abstract_any)
         registry.register(my_any, "my_any_plugin")
         res.register(registry.plugins)
 
+    # Don't allow concrete types in abstract algorithm signature
+
+    @abstract_algorithm("testing.abst_algo_bad_return_type")
+    def abst_algo_bad_return_type_1(x: int) -> Concrete1:  # pragma: no cover
+        pass
+
+    with pytest.raises(TypeError, match=" may not have Concrete types in signature"):
+        registry = PluginRegistry("test_register_errors_default_plugin")
+        registry.register(abst_algo_bad_return_type_1)
+        res_tmp = Resolver()
+        res_tmp.register(registry.plugins)
+
+    @abstract_algorithm("testing.abst_algo_bad_parameter_type")
+    def abst_algo_bad_parameter_type_1(x: Concrete1) -> int:  # pragma: no cover
+        pass
+
+    with pytest.raises(TypeError, match=" may not have Concrete types in signature"):
+        registry = PluginRegistry("test_register_errors_default_plugin")
+        registry.register(abst_algo_bad_parameter_type_1)
+        res_tmp = Resolver()
+        res_tmp.register(registry.plugins)
+
+    @abstract_algorithm("testing.abst_algo_bad_return_type")
+    def abst_algo_bad_return_type_2(x: int) -> List[Concrete1]:  # pragma: no cover
+        pass
+
+    with pytest.raises(
+        TypeError, match="return type may not have Concrete types in signature"
+    ):
+        registry = PluginRegistry("test_register_errors_default_plugin")
+        registry.register(abst_algo_bad_return_type_2)
+        res_tmp = Resolver()
+        res_tmp.register(registry.plugins)
+
+    @abstract_algorithm("testing.abst_algo_bad_parameter_type")
+    def abst_algo_bad_parameter_type_2(x: List[Concrete1]) -> int:  # pragma: no cover
+        pass
+
+    with pytest.raises(
+        TypeError, match='argument "x" may not have Concrete types in signature'
+    ):
+        registry = PluginRegistry("test_register_errors_default_plugin")
+        registry.register(abst_algo_bad_parameter_type_2)
+        res_tmp = Resolver()
+        res_tmp.register(registry.plugins)
+
+    @abstract_algorithm("testing.abst_algo_combo_combo_bad")
+    def abst_algo_combo_combo_bad(x: Union[List[int], List[float]]) -> int:
+        pass
+
+    with pytest.raises(
+        TypeError, match="Nesting a Combo type inside a Combo type is not allowed"
+    ):
+        registry = PluginRegistry("test_register_errors_combo_combo")
+        registry.register(abst_algo_combo_combo_bad)
+        res_tmp = Resolver()
+        res_tmp.register(registry.plugins)
+
+    @abstract_algorithm("testing.abst_algo_combo_combo_good")
+    def abst_algo_combo_combo_good(x: Optional[List[int]]) -> int:
+        pass
+
+    @concrete_algorithm("testing.abst_algo_combo_combo_good")
+    def my_algo_combo_combo_good(x: Optional[List[int]]) -> int:
+        return 12
+
+    registry = PluginRegistry("test_register_good_combo_combo")
+    registry.register(abst_algo_combo_combo_good)
+    registry.register(my_algo_combo_combo_good)
+    res_tmp = Resolver()
+    res_tmp.register(registry.plugins)
+
+    @concrete_algorithm("testing.abst_algo_combo_combo_good")
+    def my_algo_combo_combo_not_so_good(x: Union[List[int], List[float]]) -> int:
+        return 12
+
+    with pytest.raises(
+        TypeError, match="Nesting a Combo type inside a Combo type is not allowed"
+    ):
+        registry = PluginRegistry("test_register_good_combo_combo")
+        registry.register(abst_algo_combo_combo_good)
+        registry.register(my_algo_combo_combo_not_so_good)
+        res_tmp = Resolver()
+        res_tmp.register(registry.plugins)
+
+    # Return type cannot be a Combo
+    @abstract_algorithm("testing.combos")
+    def abst_combos(x: Union[int, float]) -> int:
+        pass
+
+    @concrete_algorithm("testing.combos")
+    def conc_combos(x: Union[int, float]) -> Union[int, float]:
+        return x
+
+    with pytest.raises(TypeError, match="return type may not be a Combo"):
+        registry = PluginRegistry("test_register_combos")
+        registry.register(abst_combos)
+        registry.register(conc_combos)
+        res_tmp = Resolver()
+        res_tmp.register(registry.plugins)
+
+    # Optional flag of a Combo must match what the abstract definition
+    @concrete_algorithm("testing.combos")
+    def conc_combos2(x: Optional[Union[int, float]]) -> int:
+        return x
+
+    with pytest.raises(TypeError, match="does not match optional flag in"):
+        registry = PluginRegistry("test_register_combos")
+        registry.register(abst_combos)
+        registry.register(conc_combos2)
+        res_tmp = Resolver()
+        res_tmp.register(registry.plugins)
+
+    # Subtypes of a Combo must match those in the abstract definition
+    @concrete_algorithm("testing.combos")
+    def conc_combos3(x: Union[complex, str]) -> int:
+        return x
+
+    with pytest.raises(TypeError, match="not found in mg.Union"):
+        registry = PluginRegistry("test_register_combos")
+        registry.register(abst_combos)
+        registry.register(conc_combos3)
+        res_tmp = Resolver()
+        res_tmp.register(registry.plugins)
+
 
 def test_union_signatures():
     class Abstract1(AbstractType):
@@ -306,12 +459,6 @@ def test_union_signatures():
     def typing_union_mixed(a: typing.Union[int, Abstract1]) -> int:  # pragma: no cover
         pass
 
-    with pytest.raises(TypeError, match="Cannot mix"):
-
-        @abstract_algorithm("testing.mg_union_mixed")
-        def mg_union_mixed(a: mg.Union[int, Abstract1]) -> int:  # pragma: no cover
-            pass
-
     registry = PluginRegistry("test_union_signatures_bad")
     registry.register(Abstract1)
     registry.register(typing_union_mixed)
@@ -332,9 +479,10 @@ def test_list_signatures():
     def typing_list_types(
         a: typing.List[int],
         b: typing.Optional[typing.List[float]],
-        c: typing.Union[typing.List[int], typing.List[float]],
+        c: typing.Optional[typing.Union[int, float]],
         d: typing.List[Abstract1],
         e: typing.Optional[typing.List[Abstract1]],
+        f: typing.Union[int, None],
     ) -> int:
         pass  # pragma: no cover
 
@@ -342,9 +490,12 @@ def test_list_signatures():
     def mg_list_types(
         a: mg.List[int],
         b: mg.Optional[mg.List[float]],
-        c: mg.Union[mg.List[int], mg.List[float]],
+        c: mg.Optional[mg.Union[int, float]],
         d: mg.List[Abstract1],
         e: mg.Optional[mg.List[Abstract1]],
+        # This is not currently supported. If the need arises, we should
+        # add mg.UnionList[int, float] which is a Combo with a new kind=UnionList
+        # f: mg.Union[mg.List[int], mg.List[float]],
     ) -> int:
         pass  # pragma: no cover
 
@@ -366,6 +517,14 @@ def test_list_signatures():
     res_good = Resolver()
     res_good.register(registry.plugins)
 
+    sig = res_good.abstract_algorithms["testing.typing_list_types"].__signature__
+    for letter in "bcef":
+        assert sig.parameters[letter].annotation.optional
+    for letter in "abde":
+        assert sig.parameters[letter].annotation.kind == "List"
+    assert sig.parameters["c"].annotation.kind == "Union"
+    assert sig.parameters["f"].annotation.kind is None
+
 
 def test_incorrect_signature_errors(example_resolver):
     from .util import IntType, FloatType, abstract_power
@@ -394,7 +553,7 @@ def test_incorrect_signature_errors(example_resolver):
     def wrong_abstract_arg(x: Concrete1, p: IntType) -> IntType:  # pragma: no cover
         pass
 
-    with pytest.raises(TypeError, match='"x" does not have type compatible'):
+    with pytest.raises(TypeError, match='argument "x" .* is not a concrete type of .*'):
         registry = PluginRegistry("test_incorrect_signature_errors_default_plugin")
         registry.register(wrong_abstract_arg)
         example_resolver.register(registry.plugins)
@@ -403,7 +562,7 @@ def test_incorrect_signature_errors(example_resolver):
     def wrong_return_arg(x: IntType, p: IntType) -> Concrete1:  # pragma: no cover
         pass
 
-    with pytest.raises(TypeError, match="return type is not compatible"):
+    with pytest.raises(TypeError, match="return type .* is not a concrete type of"):
         registry = PluginRegistry("test_incorrect_signature_errors_default_plugin")
         registry.register(wrong_return_arg)
         example_resolver.register(registry.plugins)
@@ -428,7 +587,7 @@ def test_incorrect_signature_errors(example_resolver):
 
     @concrete_algorithm("ln")
     def specifies_abstract_property(
-        x: FloatType(positivity=">=0"),
+        x: FloatType(positivity="==0"),
     ) -> FloatType:  # pragma: no cover
         pass
 
@@ -455,7 +614,9 @@ def test_python_types_in_signature(example_resolver):
     def wrong_python_type(x: IntType, p) -> IntType:  # pragma: no cover
         pass
 
-    with pytest.raises(TypeError, match='"p" does not match'):
+    with pytest.raises(
+        TypeError, match='"p": .* does not match .* in abstract signature'
+    ):
         registry = PluginRegistry("test_python_types_in_signature_default_plugin")
         registry.register(wrong_python_type)
         example_resolver.register(registry.plugins)
@@ -464,7 +625,9 @@ def test_python_types_in_signature(example_resolver):
     def wrong_return_type(x: IntType, p: int) -> complex:  # pragma: no cover
         pass
 
-    with pytest.raises(TypeError, match="is not a concrete type of"):
+    with pytest.raises(
+        TypeError, match="return type: .* does not match .* in abstract signature"
+    ):
         registry = PluginRegistry("test_python_types_in_signature_default_plugin")
         registry.register(wrong_return_type)
         example_resolver.register(registry.plugins)
@@ -477,7 +640,9 @@ def test_python_types_in_signature(example_resolver):
     def notmatching_return_type(x: int) -> float:  # pragma: no cover
         pass
 
-    with pytest.raises(TypeError, match="return type does not match"):
+    with pytest.raises(
+        TypeError, match="return type: .* does not match .* in abstract signature"
+    ):
         registry = PluginRegistry("test_python_types_in_signature_default_plugin")
         registry.register(return_type)
         registry.register(notmatching_return_type)
@@ -541,8 +706,9 @@ def test_find_translator(example_resolver):
             assert len(trns.translators) == 1
             return trns.translators[0]
 
-    assert find_translator(4, StrNum.Type) == int_to_str
-    assert find_translator(StrNum("4"), IntType) == str_to_int
+    # cannot check Translator equality because of copy_and_bind() during registration
+    assert find_translator(4, StrNum.Type).func == int_to_str.func
+    assert find_translator(StrNum("4"), IntType).func == str_to_int.func
     assert find_translator(4, OtherType) is None
     assert find_translator(4, IntType) is None  # no self-translator registered
 
@@ -577,16 +743,14 @@ def test_translate(example_resolver):
     assert example_resolver.translate(4.4, float) == 4.4
 
 
-def test_translate_plan(example_resolver):
-    from .util import StrNum, OtherType
-
-    translator = example_resolver.plan.translate(4, StrNum.Type)
-    assert not translator.unsatisfiable
-    assert translator.final_type == StrNum.Type
-    assert len(translator) == 1
-    translator = example_resolver.plan.translate(4, OtherType)
-    assert translator.unsatisfiable
-    assert translator.final_type == OtherType
+def test_translator_utils(example_resolver):
+    res = example_resolver
+    with pytest.raises(
+        AttributeError, match='No translatable type found for "FooBar" within'
+    ):
+        res.translate(4, "FooBar")
+    with pytest.raises(TypeError, match="Unexpected dst_type"):
+        res.translate(4, defaultdict(set))
 
 
 def test_find_algorithm(example_resolver):
@@ -625,22 +789,22 @@ def test_find_algorithm(example_resolver):
     assert example_resolver.find_algorithm("testing.match_python_type", set()) is None
 
 
-def test_call_algorithm(example_resolver):
+def test_run_algorithm(example_resolver):
     from .util import StrNum
 
     with pytest.raises(ValueError, match='No abstract algorithm "does_not_exist"'):
-        example_resolver.call_algorithm("does_not_exist", 1, thing=2)
+        example_resolver.run("does_not_exist", 1, thing=2)
 
-    assert example_resolver.call_algorithm("power", 2, 3) == 8
-    assert example_resolver.call_algorithm("power", p=2, x=3) == 9
+    assert example_resolver.run("power", 2, 3) == 8
+    assert example_resolver.run("power", p=2, x=3) == 9
     with pytest.raises(
         TypeError,
         match="p must be of type MyNumericAbstractType, not MyAbstractType::StrType",
     ):
-        example_resolver.call_algorithm("power", 1, "4")
-    assert example_resolver.call_algorithm("power", 2, p=3) == 8
-    assert example_resolver.call_algorithm("power", 2, StrNum("3")) == 8
-    assert example_resolver.call_algorithm("echo_str", 14) == "14 <echo>"
+        example_resolver.run("power", 1, "4")
+    assert example_resolver.run("power", 2, p=3) == 8
+    assert example_resolver.run("power", 2, StrNum("3")) == 8
+    assert example_resolver.run("echo_str", 14) == "14 <echo>"
 
     od1 = OrderedDict([("a", 1), ("b", 2), ("c", 3)])
     od2 = OrderedDict([("c", 3), ("b", 2), ("a", 1)])
@@ -650,7 +814,7 @@ def test_call_algorithm(example_resolver):
         example_resolver.algos.odict_rev(14)
 
 
-def test_call_algorithm_with_resolver(example_resolver):
+def test_run_algorithm_with_resolver(example_resolver):
     @abstract_algorithm("testing.inc_resolver")
     def abstract_test_resolver(x: int) -> int:  # pragma: no cover
         pass
@@ -665,57 +829,68 @@ def test_call_algorithm_with_resolver(example_resolver):
     registry.register(test_resolver)
     example_resolver.register(registry.plugins)
 
-    assert example_resolver.call_algorithm("testing.inc_resolver", 4) == 12
+    assert example_resolver.run("testing.inc_resolver", 4) == 12
 
 
 def test_call_using_dispatcher(example_resolver):
-    assert example_resolver.algos.power(2, 3) == 8
-    assert example_resolver.algos.power(p=2, x=3) == 9
+    res = example_resolver
+    assert res.algos.power(2, 3) == 8
+    assert res.algos.power(p=2, x=3) == 9
+    assert res.algos.repeat("abc", 3) == "abcabcabc"
+    assert res.algos.repeat(0, 4, sep=":") == "0:0:0:0"
+    assert res.algos.fizzbuzz_club(33) == "Hi, fizz"
 
     with pytest.raises(TypeError, match="too many positional arguments"):
-        example_resolver.algos.echo_str(14, "...", "$")
+        res.algos.echo_str(14, "...", "$")
 
     with pytest.raises(TypeError, match="got an unexpected keyword argument .prefix."):
-        example_resolver.algos.echo_str(14, prefix="$")
+        res.algos.echo_str(14, prefix="$")
+
+    with pytest.raises(TypeError, match="x is None, but the parameter is not Optional"):
+        res.algos.repeat(None)
+
+    with pytest.raises(TypeError, match="does not match any of"):
+        res.algos.repeat("abc", 3, sep=12)
+
+    with pytest.raises(TypeError, match="`fizzbuzz` must be one of"):
+        res.algos.fizzbuzz_club(34)
+
+    with pytest.raises(TypeError, match="x must be a list, not"):
+        res.algos.add_me_up(42)
 
 
 def test_call_using_exact_dispatcher(example_resolver):
+    res = example_resolver
     # Call with plugin at the end
-    assert example_resolver.algos.echo_str.example_plugin(14, "...", "$") == "$14..."
-    assert (
-        example_resolver.algos.echo_str.example_plugin(14, prefix="$") == "$14 <echo>"
-    )
+    assert res.algos.echo_str.example_plugin(14, "...", "$") == "$14..."
+    assert res.algos.echo_str.example_plugin(14, prefix="$") == "$14 <echo>"
     # Call with plugin at the start
-    assert (
-        example_resolver.plugins.example_plugin.algos.echo_str(14, prefix="$")
-        == "$14 <echo>"
-    )
+    assert res.plugins.example_plugin.algos.echo_str(14, prefix="$") == "$14 <echo>"
+    # Handle unsatisfiable case
+    with pytest.raises(
+        TypeError,
+        match="Incorrect input types and no valid translation path to solution",
+    ):
+        res.algos.ln.example_plugin(15)
+    # Translations are not allowed when calling exact
+    with pytest.raises(
+        TypeError, match="Incorrect input types. Translations required for"
+    ):
+        res.algos.power.example_plugin(
+            res.wrappers.MyNumericAbstractType.StrNum("4"), 2
+        )
 
 
-def test_call_algorithm_plan(example_resolver, capsys):
-    capsys.readouterr()
-    plan = example_resolver.plan.call_algorithm("power", 2, 3)
-    text = repr(plan)
-    assert "int_power" in text
-    assert "Argument Translations" in text
-    example_resolver.plan.call_algorithm("power", 2, "4")
-    captured = capsys.readouterr()
-    assert (
-        'No concrete algorithm for "power" can be satisfied for the given inputs'
-        in captured.out
-    )
-
-
-def test_call_algorithm_logging(example_resolver, capsys):
+def test_run_algorithm_logging(example_resolver, capsys):
     from .util import StrNum
 
     with config.set({"core.logging.plans": True}):
-        assert example_resolver.call_algorithm("power", 2, 3) == 8
+        assert example_resolver.run("power", 2, 3) == 8
     captured = capsys.readouterr()
     assert "int_power" in captured.out
 
     with config.set({"core.logging.translations": True}):
-        assert example_resolver.call_algorithm("power", 2, StrNum("3")) == 8
+        assert example_resolver.run("power", 2, StrNum("3")) == 8
     captured = capsys.readouterr()
     assert "StrNumType -> IntType" in captured.out
 
@@ -725,7 +900,7 @@ def test_disable_automatic_translation(example_resolver, capsys):
 
     with config.set({"core.dispatch.allow_translation": False}):
         with pytest.raises(TypeError) as e:
-            example_resolver.call_algorithm("power", 2, StrNum("3"))
+            example_resolver.run("power", 2, StrNum("3"))
 
 
 def test_algos_attribute(example_resolver):
@@ -759,6 +934,30 @@ def test_concrete_algorithm_with_properties(example_resolver):
         example_resolver.algos.ln(StrNum("0"))
 
 
+def test_assert_equal_different_types(example_resolver):
+    with pytest.raises(TypeError, match="Cannot assert_equal with different types"):
+        str_14 = example_resolver.wrappers.MyNumericAbstractType.StrNum("14")
+        example_resolver.assert_equal(str_14, 14.0)
+
+
+def test_default_resolver(example_resolver):
+    from .util import StrNum, IntType
+
+    val = StrNum("14")
+    # Test context manager mode for a resolver
+    with example_resolver:
+        # Test translating
+        assert val.translate(IntType) == 14
+        assert val.translate("IntType") == 14
+        assert val.translate(int) == 14
+        assert val.translate(0) == 14
+
+        # Test algorithm running
+        assert val.run("power", StrNum("2")) == StrNum("196")
+        # Use more robust test due to unknown algorithm resolution choice
+        example_resolver.assert_equal(val.run("power", 2), 196)
+
+
 def test_plugin_specific_concrete_algorithms():
     r = mg.resolver
     assert hasattr(r, "plugins")
@@ -851,15 +1050,15 @@ def test_duplicate_plugin():
         value_type = int
         pass
 
-    @abstract_algorithm("test_duplciate_plugin.test_abstract_algo")
+    @abstract_algorithm("test_duplicate_plugin.test_abstract_algo")
     def abstract_algo1(input_int: int):
         pass  # pragma: no cover
 
-    @concrete_algorithm("test_duplciate_plugin.test_abstract_algo")
+    @concrete_algorithm("test_duplicate_plugin.test_abstract_algo")
     def concrete_algo1(input_int: int):
         pass  # pragma: no cover
 
-    @concrete_algorithm("test_duplciate_plugin.test_abstract_algo")
+    @concrete_algorithm("test_duplicate_plugin.test_abstract_algo")
     def concrete_algo2(input_int: int):
         pass  # pragma: no cover
 
@@ -896,14 +1095,47 @@ def test_duplicate_plugin():
         )
 
     res = Resolver()
-    res.register({"test_duplciate_plugin": {"abstract_types": {AbstractType1}}})
+    res.register({"test_duplicate_plugin": {"abstract_types": {AbstractType1}}})
     with pytest.raises(ValueError, match=" already registered."):
-        res.register({"test_duplciate_plugin": {"concrete_types": {ConcreteType1}}})
+        res.register({"test_duplicate_plugin": {"concrete_types": {ConcreteType1}}})
+
+    with pytest.raises(ValueError, match=" not known to be the resolver or a plugin."):
+        _ResolverRegistrar.register_plugin_attributes_in_tree(
+            Resolver(), Resolver(), abstract_types={AbstractType1}
+        )
+
+
+def test_unambiguous_subcomponents():
+    class AbstractType1(AbstractType):
+        properties = {
+            "divisible_by_two": [False, True],
+            "in_fizzbuzz_club": [False, True],
+        }
+
+    class AbstractType2(AbstractType):
+        properties = {
+            "divisible_by_two": [False, True],
+        }
+        unambiguous_subcomponents = {AbstractType1}
 
     res = Resolver()
-    with pytest.raises(ValueError, match=" not known to be the resolver or a plugin."):
-        res._register_plugin_attributes_in_tree(
-            Resolver(), abstract_types={AbstractType1}
+    with pytest.raises(
+        KeyError, match="unambiguous subcomponent .* has not been registered"
+    ):
+        res.register(
+            {"test_unambiguous_subcomponents": {"abstract_types": {AbstractType2}}}
+        )
+
+    res = Resolver()
+    with pytest.raises(
+        ValueError, match="unambiguous subcomponent .* has additional properties beyond"
+    ):
+        res.register(
+            {
+                "test_unambiguous_subcomponents": {
+                    "abstract_types": {AbstractType1, AbstractType2}
+                }
+            }
         )
 
 
@@ -926,7 +1158,7 @@ def test_invalid_plugin_names():
 
     with pytest.raises(ValueError, match="is not a valid plugin name"):
         registry.register_from_modules(
-            mg.types, mg.algorithms, name=invalid_plugin_name
+            mg.plugins.core.types, mg.plugins.core.algorithms, name=invalid_plugin_name
         )
 
     with pytest.raises(ValueError, match="is not a valid plugin name"):
@@ -945,6 +1177,29 @@ def test_wrapper_mixing_required():
             pass
 
 
+def test_wrapper_implied_abstract():
+    class Abstract1(AbstractType):
+        pass
+
+    class Abstract2(AbstractType):
+        pass
+
+    class Wrapper1(Wrapper, abstract=Abstract1, register=False):
+        pass
+
+    class Wrapper2(Wrapper1):
+        class TypeMixin:
+            pass
+
+    assert Wrapper2.Type.abstract is Abstract1
+
+    with pytest.raises(TypeError, match="Wrong abstract type for wrapper"):
+
+        class Wrapper3(Wrapper1, abstract=Abstract2):
+            class TypeMixin:
+                pass
+
+
 def test_wrapper_insufficient_properties():
     class TestNodes(AbstractType):
         @Wrapper.required_method
@@ -964,6 +1219,14 @@ def test_wrapper_insufficient_properties():
             class TypeMixin:
                 pass
 
+    with pytest.raises(TypeError, match="must be callable, not"):
+
+        class Wrapper1(Wrapper, abstract=TestNodes):
+            __getitem__ = 42
+
+            class TypeMixin:
+                pass
+
     with pytest.raises(TypeError, match="is missing required wrapper property"):
 
         class Wrapper1(Wrapper, abstract=TestNodes):
@@ -1090,3 +1353,10 @@ def test_algorithm_versions():
                     }
                 }
             )
+
+
+def test_signature_modifier(example_resolver):
+    res = example_resolver
+    sigmod = _SignatureModifier(res.algos.power)
+    with pytest.raises(NotImplementedError):
+        sigmod.update_annotation(int, name="x", index=0)
